'==========
'DOODLE.BAS v1.3
'==========
'Little screen drawing/recording program.
'Draw something on the screen using the mouse,
'Your drawing will be recorded and played back.
'A Hand will draw it back on the screen.
'Drawings can be saved/loaded/played.
'Coded by Dav, SEPTEMBER/2023
'
'New for v1.3
'
'             - Added Save (Press S)
'             - Added Load (Press L)
'             - Added Play (Press P)
'             - Added Undo (Press U)
'             - Added CleanImage& copy of screen to use for things.
'             - Uses a new ColorPicker& function version that
'               fixes conflict in _FULLSCREEN under windows when
'               using QB64's _COLORCHOOSERDIALOG.  The new function
'               seems to have also fixed the mouse&hand glitch, so
'               I removed the $IF WIN code since it's not needed.
'             - Changed _LIMIT speed in player to only do a _LIMIT
'               for the mouse moving stuff, not for other changes.
'               This prevents pauses when changing brush size, etc.
'             - Added flag for turning menu on/off (ShowMenu. 1=on,0=off)
'             - Added extra _PSET in first down circle draws.
'               (there was an empty pixel in the middle)

'==========
'HOW TO USE:
'==========

'Left Click to draw using the mouse
'Right click mouse to clear the screen.

'D = DRAW mode (default - drawing on screen)
'F = FILL mode (like paint - fills area with color)
'C = Change color (selects a new one)
'+/- to Change brush size.

'SPACE erases recording and starts over.

'Pressing ESC will stop the recording and start playback.
'Pressing ESC during playback will stop the playback.

'==========================================================


'=== Setup nice screen size based on users desktop size

DIM SHARED CleanImage&, UndoImage&, RedoImage&, hand&

SCREEN _NEWIMAGE(INT(_DESKTOPWIDTH * .75), INT(_DESKTOPHEIGHT * .75), 32)

'make a few copies to use
CleanImage& = _COPYIMAGE(_DISPLAY) 'clean copy of current screen (no menu showing)
UndoImage& = _COPYIMAGE(_DISPLAY) 'An undo version of screen
RedoImage& = _COPYIMAGE(_DISPLAY) 'A ReDo image of screen

_DELAY .25

hand& = BASIMAGE1&

'========
StartOver:
'========

'=== settings

BrushSize% = 3 'size of brush
BrushColor& = _RGB(0, 0, 0) 'color of brush
CanvasColor& = _RGB(255, 255, 255)
DoodleLimit = 500000 'how large doodle string can be (this is about 3 minutes)
Doodle$ = "" ' clear doodle data
drawing = 0 'no drawing anything yet
ShowMenu = 1 ' Flag for if Show top bar menu or not. 1=ON, 0=OFF
Mode = 0 '0=draw with color, 1=fill with color (paint). Default ON for drawing
hand = 1 'Flag for showing hand image or not, default is on
UndoUsed = 0 'Flag if an Undo has been used (U) (Means a Redo can be done now (R))

'=== clear all screen image
_DEST UndoImage&: CLS , CanvasColor&
_DEST RedoImage&: CLS , CanvasColor&
_DEST CleanImage&: CLS , CanvasColor&
_DEST 0: CLS , CanvasColor&

'=== main drawing loop begins

DO

    WHILE _MOUSEINPUT: WEND
    mx = _MOUSEX: my = _MOUSEY

    IF mx > 0 AND mx < _WIDTH AND my > 0 AND my < _HEIGHT THEN

        IF _MOUSEBUTTON(1) THEN

            'if beginning to draw, make undo& copy of current screen
            'And make UndoDoddle$ data...
            IF drawing = 0 THEN
                'first saving current image as an undo image
                _FREEIMAGE UndoImage&: UndoImage& = _COPYIMAGE(CleanImage&)
                'now make undo copies of doodle data
                UndoDoodle$ = Doodle$: undomx = mx: undomy = my
                drawing = 1
            END IF

            IF Mode = 0 THEN

                IF stilldown = 1 THEN
                    stepx = lastmx - mx
                    stepy = lastmy - my
                    length = INT((stepx ^ 2 + stepy ^ 2) ^ .5)
                    dx = stepx / length
                    dy = stepy / length
                    FOR i = 0 TO length
                        FOR j = 1 TO BrushSize%
                            _DEST CleanImage&: CIRCLE (mx + dx * i, my + dy * i), j, BrushColor&
                            _DEST 0: CIRCLE (mx + dx * i, my + dy * i), j, BrushColor&
                        NEXT
                    NEXT
                ELSE
                    _DEST CleanImage&: PSET (mx, my), BrushColor& 'place empty pixel
                    _DEST 0: PSET (mx, my), BrushColor& 'place empty pixel
                    FOR j = 1 TO BrushSize% STEP .36
                        _DEST CleanImage&: CIRCLE (mx, my), j, BrushColor&
                        _DEST 0: CIRCLE (mx, my), j, BrushColor&
                    NEXT
                END IF

            ELSE
                _PUTIMAGE (0, 0), CleanImage&
                SPAINT mx, my, BrushColor&
                _FREEIMAGE CleanImage&
                CleanImage& = _COPYIMAGE(_DISPLAY)

                'wait until mouse button up to continue,
                'so we don't accidently fill other areas
                DO UNTIL _MOUSEBUTTON(1) = 0: mi = _MOUSEINPUT: LOOP

            END IF

            lastmx = mx: lastmy = my
            stilldown = 1
            Btn$ = "D" 'button pressed down marker
        ELSE
            drawing = 0
            stilldown = 0
            Btn$ = "U" 'button up marker, not pressed
        END IF

        'make x position data
        x$ = _TRIM$(STR$(mx)) 'x position
        IF LEN(x$) = 3 THEN x$ = "0" + x$
        IF LEN(x$) = 2 THEN x$ = "00" + x$
        IF LEN(x$) = 1 THEN x$ = "000" + x$
        'make y position data
        y$ = _TRIM$(STR$(my)) 'y position
        IF LEN(y$) = 3 THEN y$ = "0" + y$
        IF LEN(y$) = 2 THEN y$ = "00" + y$
        IF LEN(y$) = 1 THEN y$ = "000" + y$
        'build onto string data only if StartedDrawing flag on

        DoodleTemp$ = DoodleTemp$ + (Btn$ + x$ + y$)

    END IF

    'if right button clicked
    IF _MOUSEBUTTON(2) THEN
        _DEST CleanImage&: CLS , CanvasColor&
        _DEST 0: CLS , CanvasColor&
        DoodleTemp$ = DoodleTemp$ + "C00000000" 'clear screen if right click
    END IF


    'add to image data only if changed from previous x/y data
    IF DoodleTemp$ <> OldDoodleTemp$ THEN
        Doodle$ = Doodle$ + DoodleTemp$
    END IF

    'save old data
    OldDoodleTemp$ = DoodleTemp$
    'reset info
    DoodleTemp$ = ""

    '=====================================================
    'Handle keyboard input

    keys$ = UCASE$(INKEY$)

    IF keys$ = "C" THEN
        _MOUSEMOVE mx, my
        pick& = ColorPicker&(_WIDTH / 2 - 233, _HEIGHT / 2 - 123)
        _MOUSEMOVE mx, my
        _DELAY .5
        'This maked no sense, but it fixed a glitch in windows.
        'IT'S NO LONGER NEEDED NOW, but I left it in just to save.
        '$If WIN Then
        '    Do Until _MouseX = mx And _MouseY = my
        '        mi = _MouseInput
        '        _MouseMove mx, my
        '    Loop
        '$End If
        IF pick& <> 0 THEN
            BrushColor& = pick&
            Doodle$ = Doodle$ + "K" + MKL$(BrushColor&) + "0000"
        END IF
    END IF
    IF keys$ = "M" THEN
        SELECT CASE ShowMenu
            CASE 0: ShowMenu = 1:
            CASE 1: ShowMenu = 0
                _PUTIMAGE (0, 0), CleanImage&:
        END SELECT
    END IF
    IF keys$ = "F" THEN
        Mode = 1: Doodle$ = Doodle$ + "M10000000"
    END IF
    IF keys$ = "D" THEN
        Mode = 0: Doodle$ = Doodle$ + "M00000000"
    END IF
    IF keys$ = "U" THEN

        'save current things redo things first here?

        'put undo image on screen and cleanimage screen
        _DEST CleanImage&: _PUTIMAGE (0, 0), UndoImage&
        _DEST 0: _PUTIMAGE (0, 0), UndoImage&
        'Go back to where you once belonged...
        Doodle$ = UndoDoodle$
        'Make sure current chosen color is set right (for player)
        Doodle$ = Doodle$ + "K" + MKL$(BrushColor&) + "0000"
        _MOUSEMOVE undomx, undomy
    END IF
    IF keys$ = "+" THEN
        IF BrushSize% < 100 THEN
            BrushSize% = BrushSize% + 1
            Doodle$ = Doodle$ + "B" + MKL$(BrushSize%) + "0000"
        END IF
    END IF
    IF keys$ = "-" THEN
        IF BrushSize% > 1 THEN
            BrushSize% = BrushSize% - 1
            Doodle$ = Doodle$ + "B" + MKL$(BrushSize%) + "0000"
        END IF
    END IF
    IF keys$ = "S" THEN
        dagfile$ = _SAVEFILEDIALOG$("Save File as...", "", "*.dag|", ".dag files")
        IF dagfile$ <> "" THEN
            'do file save here.
            'Add .dag if not given
            IF INSTR(1, UCASE$(dagfile$), ".DAG") = 0 THEN dagfile$ = dagfile$ + ".dag"
            dag = FREEFILE: OPEN dagfile$ FOR OUTPUT AS #dag
            savedoodle$ = Doodle$
            '=== Make simple 24-byte doodle Header with drawing size info
            Head$ = "DDV1" 'Dav's Doodle v1 marker
            Head$ = Head$ + MKL$(_WIDTH)
            Head$ = Head$ + MKL$(_HEIGHT)
            Head$ = Head$ + SPACE$(12) 'extra space for future info.
            'Add it to beginning of Doodle$ string
            savedoodle$ = Head$ + savedoodle$
            savedoodle$ = _DEFLATE$(savedoodle$)
            PRINT #dag, savedoodle$;: CLOSE #dag
            _MESSAGEBOX "Information", "File saved as " + dagfile$
            _MOUSEMOVE lastmx, lastmy
            _DELAY .5
        END IF
    END IF
    IF keys$ = "L" THEN

        dagfile$ = _OPENFILEDIALOG$("Load DAG File...", "", "*.dag|*.DAG", "DAG files", -1)

        IF dagfile$ <> "" THEN
            'load file here, check header, load doodle, go to end ...
            'Grab Doodle Data
            dag = FREEFILE: OPEN dagfile$ FOR BINARY AS #dag
            loaddoodle$ = INPUT$(LOF(dag), dag): CLOSE #dag
            loaddoodle$ = _INFLATE$(loaddoodle$)

            'Load Doodle info from header
            IF MID$(loaddoodle$, 1, 4) <> "DDV1" THEN
                _MESSAGEBOX "Information", "Invalid DAG File: " + dagfile$
            ELSE

                Doodle$ = loaddoodle$
                sw = CVL(MID$(Doodle$, 1 + 4, 4))
                sh = CVL(MID$(Doodle$, 1 + 8, 4))
                'Open screen size
                SCREEN _NEWIMAGE(sw, sh, 32)
                '=== set to defaults
                Mode = 0
                stilldown = 0
                CanvasColor& = _RGB(255, 255, 255)
                BrushColor& = _RGB(0, 0, 0)
                BrushSize% = 3
                '=== clear all screen image
                _DEST UndoImage&: CLS , CanvasColor&
                _DEST RedoImage&: CLS , CanvasColor&
                _DEST CleanImage&: CLS , CanvasColor&
                _DEST 0: CLS , CanvasColor&
                'Skip over header, load data
                FOR D = 25 TO LEN(Doodle$) STEP 9
                    Btn$ = MID$(Doodle$, D, 1) 'get marker (B,U or C)
                    mx = VAL(MID$(Doodle$, D + 1, 4)) 'get x info
                    my = VAL(MID$(Doodle$, D + 5, 4)) 'get y info
                    IF Btn$ = "D" THEN
                        IF Mode = 0 THEN
                            IF stilldown = 1 THEN
                                stepx = lastmx - mx
                                stepy = lastmy - my
                                length = INT((stepx ^ 2 + stepy ^ 2) ^ .5)
                                dx = stepx / length
                                dy = stepy / length
                                FOR i = 0 TO length
                                    FOR j = 1 TO BrushSize%
                                        CIRCLE (mx + dx * i, my + dy * i), j, BrushColor&
                                    NEXT
                                NEXT
                            ELSE
                                FOR j = 1 TO BrushSize% STEP .36
                                    CIRCLE (mx, my), j, BrushColor&
                                    PSET (mx, my), BrushColor&
                                NEXT
                            END IF
                        ELSE
                            SPAINT mx, my, BrushColor&
                        END IF
                        lastmx = mx: lastmy = my
                        stilldown = 1
                    END IF

                    IF Btn$ = "U" THEN
                        stilldown = 0
                    END IF
                    IF Btn$ = "C" THEN
                        CLS , CanvasColor&: stilldown = 0
                    END IF

                    IF Btn$ = "K" THEN
                        BrushColor& = CVL(MID$(Doodle$, D + 1, 4)): stilldown = 0
                    END IF
                    IF Btn$ = "M" THEN
                        IF MID$(Doodle$, D + 1, 1) = "1" THEN Mode = 1
                        IF MID$(Doodle$, D + 1, 1) = "0" THEN Mode = 0
                    END IF
                    IF Btn$ = "B" THEN
                        BrushSize% = CVL(MID$(Doodle$, D + 1, 4))
                    END IF
                    '_DISPLAY
                    'IF INKEY$ = CHR$(27) THEN EXIT FOR
                NEXT

                'remove header space from Doodle$ data we loaded.
                Doodle$ = RIGHT$(Doodle$, LEN(Doodle$) - 24)

                '_AUTODISPLAY
                '=== update all screen image
                _FREEIMAGE UndoImage&: UndoImage& = _COPYIMAGE(_DISPLAY)
                _FREEIMAGE RedoImage&: RedoImage& = _COPYIMAGE(_DISPLAY)
                _FREEIMAGE CleanImage&: CleanImage& = _COPYIMAGE(_DISPLAY)
                drawing = 0 'no drawing anything yet
                UndoUsed = 0 'Flag if an Undo has been used (U) (Means a Redo can be done now (R))

                _MESSAGEBOX "Information", "Loaded file: " + dagfile$
                _MOUSEMOVE lastmx, lastmy

            END IF

        END IF

    END IF
    IF keys$ = "P" THEN

        BrushSize% = 3 'size of brush
        BrushColor& = _RGB(0, 0, 0) 'color of brush
        CanvasColor& = _RGB(255, 255, 255)
        Mode = 0
        CLS , CanvasColor&
        skipping = 0
        FOR D = 1 TO LEN(Doodle$) STEP 9

            Btn$ = MID$(Doodle$, D, 1) 'get marker (B,U or C)
            mx = VAL(MID$(Doodle$, D + 1, 4)) 'get x info
            my = VAL(MID$(Doodle$, D + 5, 4)) 'get y info

            IF Btn$ = "D" THEN

                IF Mode = 0 THEN
                    IF stilldown = 1 THEN
                        stepx = lastmx - mx
                        stepy = lastmy - my
                        length = INT((stepx ^ 2 + stepy ^ 2) ^ .5)
                        dx = stepx / length
                        dy = stepy / length
                        FOR i = 0 TO length
                            FOR j = 1 TO BrushSize%
                                CIRCLE (mx + dx * i, my + dy * i), j, BrushColor&
                            NEXT
                        NEXT
                    ELSE
                        FOR j = 1 TO BrushSize% STEP .36
                            CIRCLE (mx, my), j, BrushColor&
                            PSET (mx, my), BrushColor&
                        NEXT
                    END IF
                ELSE
                    SPAINT mx, my, BrushColor&
                END IF

                lastmx = mx: lastmy = my
                stilldown = 1

                handx = mx: handy = my

            END IF

            IF Btn$ = "U" THEN
                stilldown = 0
                handx = mx: handy = my
            END IF

            IF Btn$ = "C" THEN
                CLS , CanvasColor&: stilldown = 0
                '_Display
            END IF

            IF Btn$ = "K" THEN
                BrushColor& = CVL(MID$(Doodle$, D + 1, 4)): stilldown = 0
            END IF

            IF Btn$ = "M" THEN
                IF MID$(Doodle$, D + 1, 1) = "1" THEN Mode = 1
                IF MID$(Doodle$, D + 1, 1) = "0" THEN Mode = 0
            END IF
            IF Btn$ = "B" THEN
                BrushSize% = CVL(MID$(Doodle$, D + 1, 4))
            END IF

            IF hand = 1 THEN
                IF skipping <> 1 THEN
                    'save current screen
                    page& = _COPYIMAGE(_DISPLAY)
                    'shaw drawing hand over it
                    _PUTIMAGE (handx - 44, handy - 24), hand&
                    _DISPLAY
                    _PUTIMAGE (0, 0), page&
                    _FREEIMAGE page&
                END IF
            ELSE
                _DISPLAY
            END IF

            IF INKEY$ = CHR$(27) THEN skipping = 1

            'only limit speed when mouse is moving
            IF Btn$ = "D" OR Btn$ = "U" THEN
                IF skipping = 0 THEN _LIMIT 45 'play a little faster than recorded, we usually draw slow.
            END IF

        NEXT
        _KEYCLEAR
        IF skipping <> 1 THEN
            IF hand = 1 THEN
                _DELAY .5
                'done....move hand off screen here...
                FOR xx = handx TO _WIDTH STEP 5
                    handy = handy + 5
                    'save current screen
                    page& = _COPYIMAGE(_DISPLAY)
                    'shaw drawing hand over it
                    _PUTIMAGE (xx - 44, handy - 24), hand&
                    _DISPLAY
                    _PUTIMAGE (0, 0), page&
                    _FREEIMAGE page&
                    _LIMIT 100
                NEXT
            END IF
        END IF
        _AUTODISPLAY
        '=== update all screen image
        _FREEIMAGE UndoImage&: UndoImage& = _COPYIMAGE(_DISPLAY)
        _FREEIMAGE RedoImage&: RedoImage& = _COPYIMAGE(_DISPLAY)
        _FREEIMAGE CleanImage&: CleanImage& = _COPYIMAGE(_DISPLAY)
        drawing = 0 'no drawing anything yet
        UndoUsed = 0 'Flag if an Undo has been used (U) (Means a Redo can be done now (R))
        _MOUSEMOVE lastmx, lastmy

    END IF

    IF keys$ = " " THEN GOTO StartOver
    _KEYCLEAR
    '==========================================================

    _LIMIT 30

    '==== Show Menu here....
    IF ShowMenu = 1 THEN
        COLOR _RGB(255, 255, 255), _RGB(0, 0, 0)
        LOCATE 1, 1: PRINT "(M)enu On/Off, ";
        COLOR _RGB(255, 255, 255), _RGB(0, 0, 0)
        PRINT "(U)ndo, (+/-) BrushSize:"; LTRIM$(RTRIM$(STR$(BrushSize%))); ", ";
        PRINT "(P)lay File, (S)ave File, (L)oad File, SPACE Restarts, ";
        IF Mode = 0 THEN
            COLOR _RGB(255, 255, 0), _RGB(0, 0, 0): PRINT "(D)raw  ";
            COLOR _RGB(255, 255, 255), _RGB(0, 0, 0): PRINT "(F)ill, ";
        ELSE
            COLOR _RGB(255, 255, 255), _RGB(0, 0, 0): PRINT "(D)raw  ";
            COLOR _RGB(255, 255, 0), _RGB(0, 0, 0): PRINT "(F)ill, ";
        END IF
        COLOR _RGB(255, 255, 255), _RGB(0, 0, 0)
        PRINT "(C)olor->";
        COLOR BrushColor&, _RGB(0, 0, 0)
        PRINT CHR$(219); CHR$(219); CHR$(219); CHR$(219);
    END IF


    'See if set limit is reached. If so, stop recording
    IF LEN(Doodle$) > DoodleLimit THEN EXIT DO

LOOP


END



SUB SPAINT (x AS INTEGER, y AS INTEGER, clr~&) 'Color everything in the X, Y position regardless of the border color.
    'SUB by Petr
    DIM m AS _MEM, m2 AS _MEM

    m = _MEMIMAGE(_DEST)
    W = _WIDTH(_DEST)
    H = _HEIGHT(_DEST)
    P = _PIXELSIZE(_DEST)

    SELECT CASE P
        CASE 4 '                             image is 32 bit image
            Virtual = _NEWIMAGE(W, H, 32)
            m2 = _MEMIMAGE(Virtual)
            Back~& = POINT(x, y)
            Back2~& = _RGB32(1, 1, 1)
            Empty~& = _RGBA32(0, 0, 0, 0)
            DO UNTIL a& = m.SIZE - 4
                a& = a& + 4
                IF _MEMGET(m, m.OFFSET + a&, _UNSIGNED LONG) = Back~& THEN _MEMPUT m2, m2.OFFSET + a&, Back~& ELSE _MEMPUT m2, m2.OFFSET + a&, Back2~&
            LOOP
            d = _DEST
            _DEST Virtual
            PAINT (x, y), clr~&, Back2~&
            _DEST d
            a& = 0
            DO UNTIL a& = m.SIZE - 4
                a& = a& + 4
                IF _MEMGET(m2, m2.OFFSET + a&, _UNSIGNED LONG) <> clr~& THEN _MEMPUT m2, m2.OFFSET + a&, Empty~&
            LOOP
            _CLEARCOLOR Back2~&, Virtual
            _PUTIMAGE (0, 0), Virtual
            _MEMFREE m
            _MEMFREE m2
            _FREEIMAGE Virtual
        CASE 1 '                             image is 8 bit image (256 colors)
            Virtual = _NEWIMAGE(W, H, 32)
            m2 = _MEMIMAGE(Virtual)
            Back~& = POINT(x, y)
            Back2~& = _RGB(1, 1, 1)
            Empty~& = _RGBA(0, 0, 0, 0)
            DO UNTIL a& = m.SIZE - 1
                a& = a& + 1
                IF _MEMGET(m, m.OFFSET + a&, _UNSIGNED _BYTE) = Back~& THEN _MEMPUT m2, m2.OFFSET + a&, Back~& ELSE _MEMPUT m2, m2.OFFSET + a&, Back2~&
            LOOP
            d = _DEST
            _DEST Virtual
            PAINT (x, y), clr~&, Back2~&
            _DEST d
            a& = 0
            DO UNTIL a& = m.SIZE - 1
                a& = a& + 1
                IF _MEMGET(m2, m2.OFFSET + a&, _UNSIGNED _BYTE) <> clr~& THEN _MEMPUT m2, m2.OFFSET + a&, Empty~&
            LOOP
            _CLEARCOLOR Back2~&, Virtual
            _PUTIMAGE (0, 0), Virtual
            _MEMFREE m
            _MEMFREE m2
            _FREEIMAGE Virtual
    END SELECT
END SUB

'======================================
FUNCTION ColorPicker& (xpos, ypos)

    'Update ColorPicker& Function by Dav, AUG/2023.
    'Function Returns color picked by user if one selected.
    'If no color selected before Closing, function returns 0
    'Click CLOSE to close the ColorPicker image.
    'ESC key also cancels selection and closes picker box.
    'The xpos/ypos is x/y point on the screen to place colorpicker

    '=== Save users display status
    DisplayStatus% = _AUTODISPLAY

    '=== copy current screen using _MEM (thanks Steve!)
    '=== Used this method because_COPYIMAGE(_DISPLAY) didnt always work
    DIM scr1 AS _MEM, scr2 AS _MEM
    scr1 = _MEMIMAGE(0): scr2 = _MEMNEW(scr1.SIZE)
    _MEMCOPY scr1, scr1.OFFSET, scr1.SIZE TO scr2, scr2.OFFSET

    '=== Save current PRINT colors too, restore later
    fgclr& = _DEFAULTCOLOR: bgclr& = _BACKGROUNDCOLOR

    '=== Create Colorpicker menu box
    LINE (xpos, ypos)-(xpos + 463, ypos + 243), _RGB(0, 0, 0), BF
    LINE (xpos + 2, ypos + 2)-(xpos + 463 - 2, ypos + 243 - 2), _RGB(255, 255, 255), BF

    '=== make custom palette array of 16 basic soft colors to use
    REDIM pal&(0 TO 15)
    pal&(0) = _RGB(255, 50, 50) 'red
    pal&(1) = _RGB(255, 155, 52) 'orange
    pal&(2) = _RGB(255, 255, 0) 'yellow
    pal&(3) = _RGB(52, 2207, 52) 'green
    pal&(4) = _RGB(52, 105, 255) 'blue
    pal&(5) = _RGB(0, 255, 255) 'teal
    pal&(6) = _RGB(105, 105, 207) 'violet
    pal&(7) = _RGB(100, 0, 153) 'purple
    pal&(8) = _RGB(255, 192, 203) 'pink
    pal&(9) = _RGB(204, 204, 204) 'silver
    pal&(10) = _RGB(255, 207, 52) 'gold
    pal&(11) = _RGB(204, 204, 153) 'beige
    pal&(12) = _RGB(155, 75, 0) 'brown
    pal&(13) = _RGB(128, 128, 128) 'gray
    pal&(14) = _RGB(0, 0, 0) 'black
    pal&(15) = _RGB(255, 255, 255) 'white

    '=== draw color blocks
    FOR x = xpos + 10 TO xpos + 200 STEP 56
        FOR y = ypos + 10 TO ypos + 200 STEP 56
            LINE (x, y)-STEP(56, 56), pal&(p), BF: p = p + 1
            LINE (x, y)-(x + 56, y + 56), _RGB(128, 128, 128), B
        NEXT
    NEXT

    '=== draw color selection areas
    LINE ((xpos + 246), (ypos + 10))-((xpos + 453), (ypos + 70)), _RGB(128, 128, 128), B
    COLOR _RGB(128, 128, 128), _RGB(255, 255, 255)
    _PRINTSTRING (xpos + 246, ypos + 10), " New Color: "
    LINE ((xpos + 246), (ypos + 77))-((xpos + 454), (ypos + 135)), _RGB(128, 128, 128), B
    _PRINTSTRING (xpos + 246, ypos + 77), " Gradient: "

    '=== draw CLOSE button area
    w& = _RGB(255, 255, 255): r& = _RGB(255, 0, 0)
    LINE (xpos + 246, ypos + 158)-(xpos + 453, ypos + 229), r&, BF
    bx = xpos + 250: by = ypos + 158
    LINE (bx + 17, by + 11)-STEP(29, 49), w&, BF 'C
    LINE (bx + 29, by + 20)-STEP(6, 31), r&, BF
    LINE (bx + 35, by + 31)-STEP(11, 10), r&, BF
    LINE (bx + 57, by + 11)-STEP(12, 49), w&, BF 'L
    LINE (bx + 57, by + 50)-STEP(20, 10), w&, BF
    LINE (bx + 87, by + 11)-STEP(28, 49), w&, BF 'O
    LINE (bx + 98, by + 23)-STEP(6, 27), r&, BF
    LINE (bx + 125, by + 11)-STEP(26, 49), w&, BF 'S
    LINE (bx + 135, by + 20)-STEP(5, 11), r&, BF
    LINE (bx + 135, by + 27)-STEP(16, 4), r&, BF
    LINE (bx + 125, by + 39)-STEP(16, 4), r&, BF
    LINE (bx + 136, by + 39)-STEP(5, 11), r&, BF
    LINE (bx + 161, by + 11)-STEP(21, 49), w&, BF 'E
    LINE (bx + 173, by + 21)-STEP(9, 10), r&, BF
    LINE (bx + 173, by + 39)-STEP(9, 11), r&, BF
    '====================================


    '=== Now get users color selection...

    '=== no selection made yet
    selected = 0

    '=== main loop
    DO
        '=== Get mouse input
        WHILE _MOUSEINPUT
            '=== Get mouse x/y
            mx = _MOUSEX: my = _MOUSEY

            '=== Only poll this area
            IF mx > xpos AND mx < (xpos + 473) AND my > ypos AND my < (ypos + 243) THEN
                '=== if click button in area
                IF _MOUSEBUTTON(1) THEN
                    '=== if clicked in CLOSE box area
                    IF mx > (xpos + 246) AND mx < (xpos + 453) AND my > (ypos + 158) AND my < (ypos + 229) THEN
                        EXIT DO
                    END IF
                    '=== made a color selection
                    selected = 1
                    '=== Get color where mouse pointer is
                    clr& = POINT(mx, my)
                    '=== Make Red Green Blue color values
                    red = _RED32(clr&): grn = _GREEN32(clr&): blu = _BLUE32(clr&)
                    '=== show color selected in box
                    LINE ((xpos + 246), (ypos + 10))-((xpos + 453), (ypos + 70)), _RGB(red, grn, blu), BF
                    LINE ((xpos + 246), (ypos + 10))-((xpos + 453), (ypos + 70)), _RGB(128, 128, 128), B

                    '=== Update gradient strip with color...
                    '=== ...ONLY if mouse is not in gradient strip area
                    IF mx <= (xpos + 246) OR mx >= (xpos + 455) OR my <= (ypos + 78) OR my >= (ypos + 136) THEN
                        'draw from color to whiteout
                        c = 0
                        xpc = (453 - 246 / 2)
                        FOR x = (xpos + xpc) TO (xpos + 246) STEP -4
                            LINE (x, (ypos + 77))-(x + 4, (ypos + 135)), _RGB(red + c, grn + c, blu + c), BF
                            c = c + 8
                        NEXT
                        'now draw from color to blackout
                        c = 0
                        FOR x2 = xpos + xpc TO xpc + xpos + 120 STEP 4
                            LINE (x2, (ypos + 77))-(x2 + 4, (ypos + 135)), _RGB(red + c, grn + c, blu + c), BF
                            c = c - 8
                        NEXT
                        LINE ((xpos + 246), (ypos + 77))-((xpos + 454), (ypos + 135)), _RGB(128, 128, 128), B

                    END IF

                END IF
                '=== update screen, not used for now
                '_DISPLAY
            END IF
        WEND

        '=== ESC key cancels picking and closes
        IF INKEY$ = CHR$(27) THEN
            selected = 0: EXIT DO
        END IF

    LOOP 'UNTIL INKEY$ <> ""

    '=== wait for mouse button UP to continue
    DO: mi = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0

    '=== if user selected color, say so
    IF selected = 1 THEN
        ColorPicker& = clr&
    ELSE
        ColorPicker& = 0
    END IF

    '====================================

    '=== Restore background screen as it was
    _MEMCOPY scr2, scr2.OFFSET, scr2.SIZE TO scr1, scr1.OFFSET
    _MEMFREE scr1: _MEMFREE scr2

    '=== Restore display status as it was
    IF DisplayStatus% = -1 THEN _AUTODISPLAY

    '=== restore PRINT colors
    COLOR fgclr&, bgclr&

END FUNCTION


FUNCTION BASIMAGE1&
    v& = _NEWIMAGE(357, 600, 32)
    DIM m AS _MEM: m = _MEMIMAGE(v&)
    A$ = ""
    A$ = A$ + "haIkM7STLSLV66N?2j2X8FGM0TQLE4[W\6G5CXEfdLLD4DVdL_1j<\6[QdAH"
    A$ = A$ + "^FLWD<b1X11Z2X:0lU9`SaC`VLj^IgEWo_Cb\#EeGnjGn?n2000000000000"
    A$ = A$ + "00000000000000000000000000004bOncoS_n7n[om_n3oh?m_EnmYoJ2PkX"
    A$ = A$ + "BomgnkoKO7]I0>OUfkdNLI?cSY=3`iHJC^dPoSo`ok_N>6egn?mGnGJcGLOj"
    A$ = A$ + "[o\On2Q=n\;Zk;^fS7[dV;dVkOOjcolGWZlcdb_FoiLi]Coe9L7EV5;=gYk?"
    A$ = A$ + "^^ghRbOnoioeoggM67JcmQY]gJgM=ZOLYoN0^CZGg5CgSL]7GncWkn1\XlO?"
    A$ = A$ + "mGo<_a]hJOM;mhY]idOo0g9UnkdNLY5?gnSWZlnJV]M<^3_Wl_;2>?eo=]Sg"
    A$ = A$ + "OlJjaSg_\^LFCgGlSfc[m;3]_jNU[cOCk^U6mSjdf_L67enReUQ\6Oom=GkM"
    A$ = A$ + "LCNYf\IfScd[ERcZ1[;3iDVeVknd[fS7gRWk?CGNooia[dO?SV]O637_bLfB"
    A$ = A$ + "W=ah6lSjcEVIOmO6L4=ejmCP^<dGFZ;?GgMlII<f_igm?=c^aKc?bcTh8jaE"
    A$ = A$ + "Y_]3^jZG_bKY;oXFmM_;?gSK^Gma`A;j^ocGh<\dcGLjbO_g]W]gma4kEn:o"
    A$ = A$ + "cGX5Dgof[fT;WMaEjlUC_GdFGYgoU?oU^mW_#?Xn_]nMg[ld^L[dVOefCjnG"
    A$ = A$ + ";[EnI;LEEVa:MXm\;WHfmMfoghW;VdTeUQhnUo=Y;eUgBKNliILFkGnE^F6d"
    A$ = A$ + "PO?okca8mJFhRkCobDfF_6iNVbefaAfUgB?N^g_YkAVecNUQcaNM>6e?^SHo"
    A$ = A$ + "b_ceikLmG=iG[9[;3WSBSJjSWjGM_b5kecMA]`c34<`[9=`iI?>OiakI^^gZ"
    A$ = A$ + "Gi[UFiiU7V_;Kob`ihAGWL_bS]TO];oi_\hSk]UT^m`bL>6`iiM_nUWnl9gJ"
    A$ = A$ + "W_eRk;N<U0WWBKLJOmEJcYk6LlMIkG6>7em\n>WaLjV1jb`EbNllFDjV1WCG"
    A$ = A$ + ">mJEh^HY7;fja<^9[;3WW]nJEbMX;knSLnfLjeZ`MaS>7SUJcYK4G5md]U>K"
    A$ = A$ + "Ihll\kg_YmidmQdFc]1cMKedo\NkfBMIhllXW[?Wnh;9MKX5lXcdiAoI;mO_"
    A$ = A$ + "GX;3WWe\OiN[Q\W=hYonbcGNceFW[E?mm]GogMh\UQcEY;_d[kZkeSGTN[3]"
    A$ = A$ + "f^kB?6KVka2I?ON#f[<L^NdnUGkIY^DGKYOoIfGWkgoX_V7_GgijYegNlNQW"
    A$ = A$ + "KOaGUVLQW7iPcccJb[]Q<gmWec>ceef;GCWmIoo8Vk[SU>_QWMoO^dc[CGik"
    A$ = A$ + "gDkG6>NeNl[Nn5[mOCoF]V?oCo_?ng?m][onYMdUJ\;mkg3YK[jb#kYl_5]n"
    A$ = A$ + "jdaJfKgLMhjn:W[M=^a?M?T7E7OjNFGcI9Lfjem8?^9[;3k[b<e?lSodo^Kn"
    A$ = A$ + "Xm7o\^lN]GbmZ1W^gN7JbojNUmlk9\K6oh9NlNMOeVlNeZNDO>M;ECNVm;oE"
    A$ = A$ + "kG6f3CKb[mnfJjidNTM[d]C=iE^Oi6Hm<dkVnl^ch^lJk:Gi^YV\^<LFZ?VZ"
    A$ = A$ + "7gR[mheNnbja_O?n:eVeUQGgBkANZdm[[Xa]h[FCFG6N=S_NTO4k5No]gMhF"
    A$ = A$ + "jinIMIheDOn]hIMI=im_7_dSeU[bIJX;3KgJfWLjneEdSN<:V^UZ;3i<maBm"
    A$ = A$ + "dW7MdT?^Vlhfldm:W^Un8Km[?MIHmVKOb>WRc[;o8YK_f_<Tad[5>=iSehKS"
    A$ = A$ + "W^oh[H?NJK>mjMXe]VW;hHol:??NT^U^7:oon;dUQ7[n_[L_J>CW5Cg0KE[i"
    A$ = A$ + "i_^[DCfnUQeK]_fEoXfkShl#V_7G_]YNKok^OiFjJDnIlJE2lH?keR_Y]hGI"
    A$ = A$ + "n_naW^9f2^[WQadoO8fc<<_bceR?k<<f[mSU^7f:ZGcQSOkY_FnMEkajb`:l"
    A$ = A$ + "i_\HcL_W5MN6coj_bMJObCKcaGo3=X?m;C6UcjK^NlALNUYkRYKbSoo==nJT"
    A$ = A$ + "[nogj^dUM=I0?F];?M_<?jg_G]kd]b4MiaFjl;^;MI]IHNSK[WmmYOjFiIeS"
    A$ = A$ + "7ohJOj[c\gUFlDjb`2nlGRNmEU^INV]iYMhUjbgUf\cb0nNem[TLf<M_l<=^"
    A$ = A$ + "7GomceVCg;?b>LmajOU^<<X5JbgeflBkC>m?7>cnLmGeUQ_YUJbSMG^>j^cW"
    A$ = A$ + "Madn\cHVk^F\7GjDCkbCogkO5<ml;^K]hjIH<fhfLjI38QF[7?^;_dn8CgB?"
    A$ = A$ + "Zm9?mJS;m?3>kfLm]eUi^JYm8g2MjWm_[oZdV[MhbgcCoekZYMI]I^;Jac]H"
    A$ = A$ + "9Ge^lhVlLmi[^YGo5cmW[;cMA;_?i]dUke6mL?fANUKGNf?_FZke3dViF9lS"
    A$ = A$ + "E4=i_oe\fYOo^Gm\Nfhcb`iI`EE?M^5KY=W^cnZ]ia?V[gS6lEY7[;cEGM]l"
    A$ = A$ + "BWIA?dY_BkE^ZNMGDonKlgW_BCnZJje>Wf<mVa=hWeKGkkG;HJgZgKcCKbgU"
    A$ = A$ + "kS_g]9Goo3Y;C[kEj[m#;N^flEZ;?mJE>m]_]XijbOHOcOEOVfAI=Iii4i]f"
    A$ = A$ + "GkaNlE];GoeYO?VnfiF`SjaSkbUkojdcRLO]7Wi#mS]7KcG][7SYOmZ7_]V\"
    A$ = A$ + "cIVTZ_VWFL7^?N^;Mie\7jkli>[=C;CCnndU[=IW]ajkbN^cPcVNlfJcYKZ7"
    A$ = A$ + "ASn>MO2^UF\m;C2mhI0W\9O5kbefLjKkeTi^[G^NReU?fFlMl\UOefLjIF^f"
    A$ = A$ + "dSkg^LmikRg]1OgJaSmbkIfe`<74WUlSejMiJKnAOMDon8m]U]TY?gNHobd:"
    A$ = A$ + "\?iW[UkbSOn7JYWOhHNCg;bFkbU7KEYWOiji8JbG]mM_VVLb^lLW7\^l[_fe"
    A$ = A$ + "nU9YefTWnh<XHYfkE[9?nfPF\;_W?g3]d7oM]aoXNLmo^^<k]iJXceP[k9[N"
    A$ = A$ + "6Ug]VldK?JQ^ldOflZWOadON_dOfM`hN[[C>BXL^HceMND;8mLcA[LkbSjG]"
    A$ = A$ + "JKNZ7mmhdnKedGo[_CmhYL66TBIMedic7e0^bk5Nj]=?ZWeJMiJ?M=_N#=G?"
    A$ = A$ + "nEJkGI[Y7GNo\GIfKUm:?MnnAcoGi[GSi^miEkbWICNjII<gG[COkUnn8m?3"
    A$ = A$ + "f;kajdi>3iYoIeo^B?7ceBY;_V>Bji\c`Lgn<gkG[^OIkgmHY9cIYNM^Z;O="
    A$ = A$ + "f_ldc8>m]YGE>GI>:Um:G_fZ^cmhRU^=J^ggF^;?n]COKjEWV<7Qj[oCJb;O"
    A$ = A$ + "KdL_o]hiH<GS>m][GIJbLTZGSLJbcK^K;JiVld_NWkni;m]im>=I>JU^lLkI"
    A$ = A$ + ">mJocbc^mI^?VF_;?N?ccM]a=nkQdgngSdUi<DKcYG_OfFc]=cma]fcaXEJd"
    A$ = A$ + "?ZG?m]CocTFWV<W][leVlD[mfTinHkeVlJfC=;CCV4^;Mi]LKbLOlmLGfe#g"
    A$ = A$ + "[ACV4^;=iR]LkbLOlmfiH\dI<o<SOL^=gOnMQV<9LWJb5KifVinhKm^lALM<"
    A$ = A$ + "OgJaEJbTbMZ;_eKKVkcA;gUGK7N^OV?mi1Rdo\:==IBi>eT;fj]?cmiX5jbS"
    A$ = A$ + "gClhNlFgO\nkgcc?LTDMmGjiPcb[LKdLONJ]^\k3_mSN<YMg^VUOUKSVkccF"
    A$ = A$ + "kb7AKN=?7;cfX9CJgYm9GlZg>=gW[FX;?N?ce_VBOKL_biF#;h^eT;NeK[Vn"
    A$ = A$ + "iYEJb?ZE[A_NUoM7YW7QRdc2W]mlfZF_;?^??m[kiNkk\bihUNF4:^KkEnM_"
    A$ = A$ + "mZN>1S]gMienJC`kLO3>ModS=nk[dolZGFG0k5=i]kMKb?[=?ne:e4kYn>eR"
    A$ = A$ + "eTYeLgJb7EGnEKc[i<7>kcoh^eTM^5dJ^KMimhf\cX9?NO[eccHJWfnTKSe3"
    A$ = A$ + "`Nj^eTka^LmeIaWmgGim_lWKObJbdgB?Gd[cQ;mio8Jce^lLgOL?bdkg^Ymk"
    A$ = A$ + "[RGikWdcM`B:kEn>mLP`NMkfBOngkkg_WM6cY_m\5X9cERc_helX?o7AGNJC"
    A$ = A$ + ">m]Q]4=I^JB?CMf>SK_Ne^lSJcceSeWc\60>BYW?jYIb]ngcNgUmjjd[k7na"
    A$ = A$ + "Oj]oi?L6^b??amXneF_MjEnkmMf_lL]ijO^fl[;ml6l<gacDnEV=OeoN>Z^l"
    A$ = A$ + "h?oY_]\WTNN3NVkJCN;cW_kO?kmi;_7Gk5geFNji=H=dUGfNmgcAeUgjG3Y_"
    A$ = A$ + "mn\=nLTGc?_QF`M^9?gLjAmgcNO>6KI?cSOO6oaWnfncBY=WN>3fRkNGn\LT"
    A$ = A$ + "Mielgkdgn^#CVNSVlihMKb?Z;_e_>BOKaI`e07m:=iU]WoKlW^Gem\;OW>;R"
    A$ = A$ + "GAjI<H[^:MiFnkSbc_8eGK>?Z^lMI?`KAjI;hEdb]\]ZUoNAGnLi\TYGebM\"
    A$ = A$ + "[TjI<DO>ikX>SiJKn^OV6JbdcdU?GeW_D?ZVlMOOcUNLiaRNji:hEY9O^:mh"
    A$ = A$ + "mJobjbcc[mNdcndG_G??gWfhK[?bc`h>gUC?C1_]?oUhcAgAkaIH\f^lMh<V"
    A$ = A$ + "[GKbaWWPM#jiYkXjeSAi]?SfLj_O?3YWSPmBjIYkXYWIdAgUComkI8mL4\G^"
    A$ = A$ + "b?Ggg;fSkk_eN>6U[=QjcEnSoHkmcO>mL4\WB??MWE_N<fS7G9?[=?moNigG"
    A$ = A$ + "kccmJJBmcIjKSNVdc?`NcNUcYfS?S7G9SEK`SOkY=hNX7GTNn1fK^FUcJli;"
    A$ = A$ + "GgglAgTODk]gjbYWOPSP^LKXnlSAi]?Rm;Go_EfKldko_YmgNin7<m\3LDB?"
    A$ = A$ + "Ka7ON;jXf_lLWCaHfW<d6B?KaPchiC^NXgn<YWIPSTk[_faNN]H<GOFC6j3Y"
    A$ = A$ + "WahSVN>2kEGFC6j3f[LkIle;gNgTki^\W_Ti>#C^m\Wg_OC=gSETNAjI5h\T"
    A$ = A$ + "NFSi]Gg_O;9mgOKEji4h\hJENKMbcn_cJolM?3SaoZV<d^Ln5]]mheNeYWY\"
    A$ = A$ + "V<dfB?_ab6gTg[[QiN[=WNn0>Kf[LkkX>OidOO]6YW?P4B?gac=ni7SRkBK>"
    A$ = A$ + "m\1TPk[_n`Nnj_jdcaXL]bUnk_YB?G0YX9gWfcm:_d[SD9O>S;mL1TddGkhX"
    A$ = A$ + "]Md?g5Enk8m[cOYW9PT\GinbAnl_Lj_g:B??0Yij_X_DOM:Ynkg[^Lj__ZB?"
    A$ = A$ + "?0]PdcQljn=o^oiUZ;WNF0J1f[LoJ_N]G]5JbYW3PFQcD^?EO>T[nkkmm;WN"
    A$ = A$ + ">0J5SO]^Wggh>IZonka^Lji0XUX9OMd[kG>m<0dBLVbG;;eKOdo]dWaLjI0X"
    A$ = A$ + "UX9OmlXNkcn_Of=id[oQFSVleckN66WiiHTNm?dRLmGLMebG[LYGg3]:kEnJ"
    A$ = A$ + "J==g]ln]gB_^7JIYk7LLfScbh8jbYGc3]<WOaedNeSgkcb8mjMXeY9O?d:kI"
    A$ = A$ + ">mjMX7X;Om]WkInMnj8mJMX7X9O?DOnb8IG>mJMX7X9O?\g_^Ho:O=TN]>d;"
    A$ = A$ + "dU_n:kCnI_VUDOO?R^Ljeh#?ACnni\g_Ljeh#?ACnNI=WSAmmM_jbYGS3m6M"
    A$ = A$ + "ikSeNfbegomH?cYGO3m6?F[_?VnlWmJJ\_KG>mjKX7U^E`ikIkI^nnDneefU"
    A$ = A$ + "Wnhn;mjJXGI_bg?e[7SUjbUgWaW[e[\Oid[[QNUVlmDY;oXGC\Wnn_e^Ljee"
    A$ = A$ + "#?;MOPlfkcG>mJJXWie?EZfSm<GNOB_V6jIJbD]DSM;kI^nnTNM=dcBg2X]\"
    A$ = A$ + "7WSaOh_ngeUQGDj60]QeLMbEOo:ojcJbUgWd[]QNDjF0]QJC^lfkAGfi;3_^"
    A$ = A$ + "dm0bKlm]`SJ^SOOLma17Sdm0J7SOLUDnmf_<LnBg1X]DOLU\V77f[Y;GolVN"
    A$ = A$ + "M>d;lHjSYFcc;n5[];Kob`fT^1#KI^[MmgMobjb`jI_b\6_knUMMb1[GjiMj"
    A$ = A$ + "7f_<L>B?[CK[llUlc^F<NfSoTjWSbkOjek#[cIH`SDjY;eUW^OieN=<WN=?d"
    A$ = A$ + "jB?gCkCG6>?YWgY]E_V<Z_^?m\kg?MIhmhl;H;Nf[K9jb`k;mL>mQJ_]dUGc"
    A$ = A$ + "IIX;3_6kEVeXN]_E^^SOD[eiH0__dckd7ZWMAiGo=o^oiZjbUgo7eU;O^B_n"
    A$ = A$ + "7J=f[<[eJg3lFf_LAjI0XUX9cJEO^T[Nm;_f7K9jb`fX;cSDGWl3oh?mQNlc"
    A$ = A$ + "N\UhlUQGWGO[I9SGW\UVld[7SbKKob`jH_b\TY[EV[;_fm;GnlY;3[CjIOJC"
    A$ = A$ + "c]FiXjbe?7YW5PFPm:cLFJmbkdUMon1[S^<Sm\e;;eUGZ=kL<P];M7PfaJF_"
    A$ = A$ + "\U^Lmc[^<\Nf[<5KI=cFg_Lic_^<\N^fhH[[I>bkg_Rdc4#Bf[<_bjVShlU7"
    A$ = A$ + "oi8mL1TDjV0mGC^hAMiiJcSkb;M=IDohdUi^;MGPn[9Gl\^ldnLi__edU]OI"
    A$ = A$ + "#Gn^jMGg\UVlJjbCoHC?G09U^?#oeT;fJGNlIIDnJH^F\V<l=YK4dO=iReN>"
    A$ = A$ + "6EU[iWYggOJb`g;MSPn\9G\UVlLGobJb`lBg9X?Kb5UG?ofjNVGZ7[93OCj>"
    A$ = A$ + "1mI?^J[=ImHhiBg;XOKb5ja`nbSa_[_SN=TN<\_dU_f>Se#KmlUC_V7JMYk6"
    A$ = A$ + "dg=iRefUC_F7j5YK7dg=iRWeUC_67j=Yk7dg=iRUjbYGK3mZd=4jkV<`n;M7"
    A$ = A$ + "1=IP?:M;1=IP?:M?1=IP1o`_<AW^YPV<`POjoFG^WUNm3`n;MG1=IP?:MK1="
    A$ = A$ + "IP_Gj63Jb0ODj>3Jb03BgI#G6hSBgI#C6hSBgJ#C6H#jF3Jb0ODjN3Jb03Bg"
    A$ = A$ + "K#C6hSBgL#C6hSBgM#C6H#j^3Jb03BgMBYlJ=Gj_6dT1VCjf3Jb0ODjn3Jb0"
    A$ = A$ + "3BgO#C6H#jn3Jb0ODj64Jb03lJhGK9mj1P\nSOmo9N72=IP1Yk#X93<8M72="
    A$ = A$ + "IP1Yk#X93lAYKAX93<8M;2=IP1YKAX93lAYkAX93<8M?2MIP1ce7jQWcQ_JB"
    A$ = A$ + "_>0X=T^5QV<`7U^7QV<`P]d>L^6Jb07;??aUGje00]3??aUGje00]Ud=YgeE"
    A$ = A$ + "h<EB_60XMd[WOaEX5[93<EjN4Jb03BgS#G6H#jF4Jb03BgR#G6H`Oj_l_RgR"
    A$ = A$ + "#C6hKnQOlWRgR#G6hK\?ifCje40iX9gNB_V08WolWo7aK#X93<8M32=IP1YK"
    A$ = A$ + "#X93<8M32MIP1^OnJ?YGC0TCjn3jb03\?ifDje50IlY?ocak?X93l=_ceXlE"
    A$ = A$ + "jiFnFCje50iT^oPV<`Pdm7dT16T^oP^<`0Gk5]Yd[;0b`c5A]Yd[;0b8Mk1="
    A$ = A$ + "IP1YK?X93lAYk?X93<8Mo1=IP1Yk?X93<8Mo1=IP1Yk?X93<8Mo1MIP1YK?X"
    A$ = A$ + "93<`S_jfDje50Il3oh?5_oPV<`gX9gVB_^083WMAK:mj2P<BgN#C6H#jf3Jb"
    A$ = A$ + "03Lf5]Yd[;0bhCOnWSgO#C6hKdTKCYGG0TQGoWJGYGK04`WobZK4>kI=IPSe"
    A$ = A$ + "WnjjKbX93LlBgN#C6H#jf3Jb03BgN#C6H#jf3Jb03BgN#C6H`Jj3^>hdT1>7"
    A$ = A$ + "YK?X93<8Mk1=IP1YK?X93<8Mk1MIP1Yk>X93<8Mg1=IP1Yk>X93<8Mg1=IP1"
    A$ = A$ + "NN]_MUN]1`iCC^MUN]1`iCC^MUN]1`iCC^MUN]1`i;Mg1=IP1Yk>X93<`IGd"
    A$ = A$ + "^B_f0hlY9gfB_n0hTmi_4_kPV<`glY_Z9gjB_61hLU^iPV<`Pd=7dT16T^iP"
    A$ = A$ + "V<`ghj^XmUN=2`iAC^mUN=2`9be2Gc;nJ4PCSm9gnB_61hlX9oNn]onoVV<`"
    A$ = A$ + "^:MG[WY93\gBge#G6H#jN3Jb03BgK#C6H#jN3Jb03BgK#C6H#jN3jb03nCoU"
    A$ = A$ + "oElN3Jb0OSVLO8mj4PcAjF3Jb03lh[^?TNM2`i#CneM6?fZeTQK6?OMf5R_>"
    A$ = A$ + "1hDHObmQd[C0>7OjcoLlN3Jb0OSVLo8mJ5PSW[?inAje:07_dM6dU16T^aPV"
    A$ = A$ + "<`Pd=6dT1nV?mEGKbk9_nl1l^Bga#C6H#j63jb03BgGjIWicg5Jb`m#j^6Jb"
    A$ = A$ + "03BgGjM9f[Lje<073?6njgfLje>0k?=inEje>0k?=inEje>0k??g9ggB_n1H"
    A$ = A$ + "OY9ggB_n1HOii>inFje?0k;kC^_UNm3`>c[?ZM_h[Q0f=fWle#jeA0k3WWlk"
    A$ = A$ + "9ecg5Jb`eTm9[93d>dT_6B_>2HoT^WP^<`Pd]4dT16T^UPV<`Pd]4dT1n6g7"
    A$ = A$ + "OG7YG;1lndT_>B_F2hmU^SPV<`PdM4dU16T^QPV<`Pd=4dT16T^QPV<`Pd=4"
    A$ = A$ + "dT16T^QP^<`Pdm3dT16T^OP^<`Pd]3dT16T^MPV<`Pd]3dT16T^MPV<`Pd]3"
    A$ = A$ + "dT16T^MP^<`PdM3dT16T^KPV<`PdM3dT16T^KPV<`PdMSNg_mgogRoePV<LM"
    A$ = A$ + "T^KL5dZMid[]0fVF_YP^<LWT^GPV<`7U^IP^<`Pdm2dT16T^GPV<`Pdm2dT1"
    A$ = A$ + "6T^GP^<`Pd]2dT16T^EPV<`Pd]2dU16T^CP^<`gT^?PV<`Pdm1dT16T^?PV<"
    A$ = A$ + "`Pdm1dT16T^?P^<`Pd]1dT16T^=PV<`PiVGmJ>beGjeM0c;MK0MIP1c=[FfW"
    A$ = A$ + "\m:OmUN]7`gKYiE=i[_d[m0nNceN[kCFGnJ;mJ?P?JYIea=IMi[[d[o0nX7="
    A$ = A$ + "_Z7O?TN=8`Po`Om_odV\^leFjeP03Z]g7=cZ9OmUNM8`gDK^j^TN]8`jJbje"
    A$ = A$ + "g3YG;2gMoYobojGK^j^TNm8LgEKb5jbTNm8Lg=NNDCVd[7Qk^olWo7ojlXN<"
    A$ = A$ + "5YGC2gIYWoYmTN=9LWUNnWfCjeT`MFjiOJ?YGC2gES?;IX:mjBh^ZLM`ineS"
    A$ = A$ + "YB_^4^[B?kCK:mjBh^:m\?]Yd[;QkZdcndVB_^4^[FclYcKnN:mJCh>jIcUj"
    A$ = A$ + "agKYGO2g=YWiY]UNm9LgTNVWfFjeW`MCI^ciC`STN=:LWTNNWfGjeX`MbhW?"
    A$ = A$ + "kQiTN=:LGhij2F[d[EQk0kAVe:mJEh>#CV]8mjEhZ;m<>mUd[GQ[^dchdGB_"
    A$ = A$ + "N5^jB?SCo9mJFh:K;cRU7G9N\U#je\`EFjiKj?YGc2GIYW_Y?UNM;LEUNfVn"
    A$ = A$ + "Dje]`EEjIKjGYGk2GAYW[YOUN];L5UN^VnEje^`EDjiJjGYGk2GAYW[YOUN]"
    A$ = A$ + ";L5UN^VnEje^`ESWChhMTNm;LeX9ck8mjGhZa[a8l>B_n5^JB?CCo;mJHh:9"
    A$ = A$ + "ml<m_d[QQ[TdccdoB_66^BB??Co;mJHh:9ml<m_d[QQ[TdccdoB_66^BB??c"
    A$ = A$ + "e#jea`EAjII^6B_>6^2lh5QmDjec#_;m<<G;YG?3m^dc`L]TNm<dkB?3ceCj"
    A$ = A$ + "ed#?;ml;G?YGC3m\dc_L=UNM=d[6?7mKomo]hcbL=TNM=d[V>;Y;cN8mjJXG"
    A$ = A$ + "UNfU[Yd[[QNEjIG^^B_f6jAYWKij:mJKX7UN^U[[d[]QNdSVYLomak:mjKXg"
    A$ = A$ + "lXi9=INGYGO3mVdc\LmUN=>dCFkLUm<c[:mJLXW\fi:=INEYGS3mTdc[LmUN"
    A$ = A$ + "=>dCNdnO]gHf;YGW3mRij^Ja\gB_>7j5UiUY=hj_GKVmBjei#_HYI8MIfCYG"
    A$ = A$ + "W3mRj<Sf;7]d[eQN#Y5k<Ui<TN]>d3Zc;J`LdB_F7j1eiUU^OnPmBjej#?X<"
    A$ = A$ + "[\diG0k]d[gQFGI>ACVcBjek#[[>[X9cI8mjMXeUN6UkUd[gQFGjID^OB_V7"
    A$ = A$ + "JI[M>bIK`N9mJNXUYebI;mJNXEUNfTkYd[kQFEjIC^_B_f7JAYW;in:mJOX5"
    A$ = A$ + "]dlRcHVSFjen#;Z<KXoB:YGo3]VbLacNLE[IcA9mjOXelXi5]H>JYGo3]VbL"
    A$ = A$ + "Qf;9UN60J9YW7Qdc0#;Y<ChiYCB:m<0d:n3oeonoNV`[?eTDji0XEDW9ljS:"
    A$ = A$ + "9UN>0J5eI2WM19UN>0J5[MVA_VSDji0XETNF4:B?70]RYc6f?<YTNF0J1UIQ"
    A$ = A$ + "jI9[7CJYW7PdZc2jbd2B??0YEW56gSeU9Udc3#JUi0k?VFBjI28Yb<Pc]PFB"
    A$ = A$ + "jI28Yb<`hV\^<YUNV0BZ<3X9C[9mL1TBImodV\^<YUN^0BA?VFDji28UUJbJ"
    A$ = A$ + "cTFjI38QiJbjad2B?K09X7C[:m\1TPm9C[:m\1TPF<]Zdc6#2?jnhCSVTB?K"
    A$ = A$ + "09hL;XEUNf0B`i9C[:m\1T`JjaJdTBji3h\mXV\F<YUNn0>K;Mf5>33J1YW?"
    A$ = A$ + "PcVV<]\dc7`ICCVFGjI4h<Y9C[;m<2LVNFCFKVdB?S0WYefTeU9Ydc9`IA?V"
    A$ = A$ + "N#ji4h\l^mHmJ>3YWCPcbFfWldOMjK37Udc9`IiEKaeO_V<WUdc:`IhE>g2W"
    A$ = A$ + "e<9TNF1>3KIObSY9cI;m\2L6dTYWTNN1>J_jm_WV<9TNN1>J_ce2WV<W]dc;"
    A$ = A$ + "`AjEOlRhnhSDB?c07Ygia`WV<YTN^1>:[]9_deU\V<9TN^1>:_BCN;Wg<LDB"
    A$ = A$ + "?k07Q]dTOdNUC?ObmDji7H_]eVlF_^<PSDji7H_Y9C_;m<4\WdTYgUN6J>oo"
    A$ = A$ + "R3cl%%%0"
    btemp$ = ""
    FOR i& = 1 TO LEN(A$) STEP 4: B$ = MID$(A$, i&, 4)
        IF INSTR(1, B$, "%") THEN
            FOR C% = 1 TO LEN(B$): F$ = MID$(B$, C%, 1)
                IF F$ <> "%" THEN C$ = C$ + F$
            NEXT: B$ = C$: END IF: FOR j = 1 TO LEN(B$)
            IF MID$(B$, j, 1) = "#" THEN
        MID$(B$, j) = "@": END IF: NEXT
        FOR t% = LEN(B$) TO 1 STEP -1
            B& = B& * 64 + ASC(MID$(B$, t%)) - 48
            NEXT: X$ = "": FOR t% = 1 TO LEN(B$) - 1
            X$ = X$ + CHR$(B& AND 255): B& = B& \ 256
    NEXT: btemp$ = btemp$ + X$: NEXT
    btemp$ = _INFLATE$(btemp$, m.SIZE)
    _MEMPUT m, m.OFFSET, btemp$: _MEMFREE m
    BASIMAGE1& = _COPYIMAGE(v&): _FREEIMAGE v&
END FUNCTION




